package core.schedules;

import core.api.models.timemanager.taskSchedule.BasicScheduleFieldInfo;
import core.api.models.timemanager.taskSchedule.BasicScheduleInfo;
import core.entities.timemanager.AbstractSchedule;
import core.entities.timemanager.BasicTaskSchedule;
import core.entities.timemanager.Task;
import core.repositories.timemanager.TaskgroupRepository;
import core.services.ServiceExitCode;
import core.services.ServiceResult;
import core.services.TaskScheduleService;
import core.services.TaskgroupService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.context.jdbc.SqlGroup;

import javax.persistence.EntityManager;
import javax.transaction.Transactional;

import java.time.LocalDate;
import java.util.List;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.junit.jupiter.api.Assertions.*;

@SpringBootTest
@Transactional
public class ScheduleServiceTest {


    @Autowired
    private EntityManager entityManager;

    @Autowired private TaskScheduleService taskScheduleService;

    private static final String username = "Testuser1";
    private static final String username2 = "Testuser2";

    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql"),
            @Sql("classpath:basicScheduleEntries.sql")
    })
    void getAllSchedulesOfUser() {
        assertEquals(0, taskScheduleService.getAllSchedulesOfUser(username2).size());

        List<AbstractSchedule> result_1 = taskScheduleService.getAllSchedulesOfUser(username);
        assertEquals(2, result_1.size());
        assertTrue(result_1.contains(entityManager.find(AbstractSchedule.class, 1L)));
        assertTrue(result_1.contains(entityManager.find(AbstractSchedule.class, 2L)));
    }

    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql"),
            @Sql("classpath:basicScheduleEntries.sql")
    })
    void scheduleBasic() {
        //Situation 1: Schedule finished Task
        ServiceResult<AbstractSchedule> result_1 = taskScheduleService.scheduleBasic(entityManager.find(Task.class, 2L), new BasicScheduleFieldInfo(LocalDate.now()));
        assertEquals(ServiceExitCode.INVALID_OPERATION, result_1.getExitCode());

        //Situation 2: Schedule before today
        ServiceResult<AbstractSchedule> result_2 = taskScheduleService.scheduleBasic(entityManager.find(Task.class, 1L), new BasicScheduleFieldInfo(LocalDate.of(2010, 3, 14)));
        assertEquals(ServiceExitCode.INVALID_OPERATION, result_2.getExitCode());

        //Situation 3: Valid schedule
        ServiceResult<AbstractSchedule> result_3 = taskScheduleService.scheduleBasic(entityManager.find(Task.class, 1L), new BasicScheduleFieldInfo(LocalDate.now()));
        assertEquals(ServiceExitCode.OK, result_3.getExitCode());
        assertThat(entityManager.find(BasicTaskSchedule.class, result_3.getResult().getScheduleID())).isNotNull();
    }

    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql"),
            @Sql("classpath:basicScheduleEntries.sql")
    })
    void editBasicSchedule() {
        //Situation 1: Reschedule finished task
        ServiceResult<AbstractSchedule> result_1 = taskScheduleService.editBasicSchedule(entityManager.find(BasicTaskSchedule.class, 2L), new BasicScheduleFieldInfo(LocalDate.now()));
        assertEquals(ServiceExitCode.INVALID_OPERATION, result_1.getExitCode());

        //Situation 2: Reschedule unfinished task with invalid reschedule date
        ServiceResult<AbstractSchedule> result_2 = taskScheduleService.editBasicSchedule(entityManager.find(BasicTaskSchedule.class, 1L), new BasicScheduleFieldInfo(LocalDate.of(2011, 3, 4)));
        assertEquals(ServiceExitCode.INVALID_OPERATION, result_2.getExitCode());

        //Situation 3: Reschedule unfinished task with valid reschedule date
        LocalDate oldDate = entityManager.find(BasicTaskSchedule.class, 1L).getScheduleDate();
        ServiceResult<AbstractSchedule> result_3 = taskScheduleService.editBasicSchedule(entityManager.find(BasicTaskSchedule.class, 1L), new BasicScheduleFieldInfo(LocalDate.now()));
        assertEquals(ServiceExitCode.OK, result_3.getExitCode());
        assertNotEquals(((BasicTaskSchedule) result_3.getResult()).getScheduleDate(), oldDate);
    }

    @Test
    @SqlGroup({
            @Sql("classpath:taskgroupRepositoryTestEntries.sql"),
            @Sql("classpath:taskRepositoryEntries.sql"),
            @Sql("classpath:basicScheduleEntries.sql")
    })
    void deleteSchedule() {
        for(long i=1; i<=2; i++) {
            taskScheduleService.deleteSchedule(entityManager.find(BasicTaskSchedule.class, i));
            assertThat(entityManager.find(BasicTaskSchedule.class, i)).isNull();
        }
    }
}
